"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ServiceErrors_1 = require("./ServiceErrors");
const RuntimeDebugging_1 = require("./RuntimeDebugging");
/**
 * @private Infrastructure that should never be used outside the Framer Services package implementation.
 */
var ServiceRuntimePrivate;
(function (ServiceRuntimePrivate) {
    // Capture Math.random in case some other code swaps it out
    const randomNumber = Math.random;
    /** Returns a good enough unique string to identify request/response pairs across services */
    function generateUniqueId() {
        return `${randomNumber()}`;
    }
    ServiceRuntimePrivate.generateUniqueId = generateUniqueId;
    function newResolvablePromise() {
        let promiseResolve;
        let promiseReject;
        const promise = new Promise((resolve, reject) => {
            promiseResolve = resolve;
            promiseReject = reject;
        });
        promise.resolve = promiseResolve;
        promise.reject = promiseReject;
        return promise;
    }
    ServiceRuntimePrivate.newResolvablePromise = newResolvablePromise;
    function isServiceEventStreamOptions(body) {
        if (!body) {
            return false;
        }
        switch (body.replay) {
            case "latest":
            case undefined:
                break;
            default:
                return false;
        }
        return true;
    }
    ServiceRuntimePrivate.isServiceEventStreamOptions = isServiceEventStreamOptions;
    class StreamReader {
        constructor(method, options, helper) {
            this.method = method;
            this.options = options;
            this.helper = helper;
            this.read = (callback) => __awaiter(this, void 0, void 0, function* () {
                const iterator = this[Symbol.asyncIterator]();
                const { iteration } = this;
                if (!iteration) {
                    // Iteration context should have been set up with the iterator
                    throw new ServiceErrors_1.ServiceError.BadRequest("Cannot start reading stream");
                }
                this.iterate(iterator, callback);
                return Promise.race([iteration.isDonePromise, iteration.isCancelledPromise]);
            });
            this.cancel = () => __awaiter(this, void 0, void 0, function* () {
                const { iteration } = this;
                if (!iteration) {
                    // Don't allow cancelling the stream before reading started
                    throw new ServiceErrors_1.ServiceError.BadRequest("Cannot cancel a stream before reading it");
                }
                iteration.isCancelledPromise.resolve();
            });
        }
        iterate(iterator, callback) {
            // Note: this uses explicit Promises to work around a strange crash in JavaScriptCore at time of writing
            // See https://github.com/framer/company/issues/13411 for more info
            void iterator.next().then((result) => __awaiter(this, void 0, void 0, function* () {
                if (result.done) {
                    return;
                }
                yield callback(result.value);
                this.iterate(iterator, callback);
            }));
        }
        [Symbol.asyncIterator]() {
            if (this.iteration) {
                throw new ServiceErrors_1.ServiceError.BadRequest("ServiceStream instances can only be read once. If multiple AsyncIterators or read() calls are required, create a new stream for each by calling the associated service method. To broadcast events with an observer pattern, consider using a client-specific EventEmitter or similar.");
            }
            const iteration = (this.iteration = {
                isDonePromise: newResolvablePromise(),
                isCancelledPromise: newResolvablePromise(),
            });
            // Contrary to what the types say, value = undefined is valid when done = true
            const doneResult = { done: true, value: undefined };
            // The stream id will let the Router on the other end remember the corresponding iterator
            const id = generateUniqueId();
            const logger = RuntimeDebugging_1.ServiceDebugging.createLogger(`🏄‍ [${id}]`);
            return {
                // Note: this uses explicit Promises to work around a strange crash in JavaScriptCore at time of writing
                // See https://github.com/framer/company/issues/13411 for more info
                next: () => {
                    const valuePromise = this.helper({
                        method: this.method,
                        argument: this.options,
                        stream: { id, method: "next" },
                    }).then((result) => {
                        if (!isIteratorResult(result)) {
                            // FIXME: reject instead of ending the iteration?
                            logger.log("StreamReader.next received an invalid iterator result for next()", result);
                            return Promise.resolve(doneResult);
                        }
                        return Promise.resolve(result);
                    });
                    return Promise.race([
                        valuePromise
                            .then(result => {
                            // The next value may "never" resolve, e.g. if an event emitter isn't emitting new events
                            if (result.done) {
                                iteration.isDonePromise.resolve();
                            }
                            return result;
                        })
                            .catch(error => {
                            // Stop iterating if an error occurred
                            iteration.isDonePromise.reject(error);
                            iteration.isCancelledPromise.resolve();
                            return Promise.reject(error);
                        }),
                        iteration.isCancelledPromise.then(_ => {
                            // Communicate the cancel to the service side, but don't make next() wait for it
                            this.helper({
                                method: this.method,
                                stream: { id, method: "return" },
                            }).catch(error => {
                                // There's not much we can do if the cancel failed on the service side or in transit.
                                logger.log("StreamReader received error trying to cancel iterator on the service side", error);
                            });
                            return doneResult;
                        }),
                    ]);
                },
            };
        }
    }
    ServiceRuntimePrivate.StreamReader = StreamReader;
    function isIteratorResult(result) {
        if (!result)
            return false;
        return result.done === true || (result.done === false && result.value !== undefined);
    }
})(ServiceRuntimePrivate = exports.ServiceRuntimePrivate || (exports.ServiceRuntimePrivate = {}));
(function (ServiceRuntimePrivate) {
    let Validation;
    (function (Validation) {
        function typeContextForError(error) {
            if (!error)
                return "";
            const { stack } = error;
            if (typeof stack !== "string")
                return "";
            // Based on the stack trace, try to figure out what type hierarchy caused the error
            const validators = stack
                .split("\n")
                .map(line => {
                const matches = line.match(/([^\.\s]*)Validation.copy([^\.\s]*)/);
                if (!matches || matches.length < 3)
                    return { namespace: "", type: "" };
                return { namespace: matches[1], type: matches[2] };
            })
                .filter(validator => !!validator.type)
                .reverse();
            return validators
                .map((validator, i) => (i === 0 ? validator.namespace + "." : "") + validator.type)
                .join("/");
        }
        Validation.typeContextForError = typeContextForError;
        function copyString(value) {
            if (typeof value !== "string") {
                throw new Error(`Expected string but received: ${value}`);
            }
            return value;
        }
        Validation.copyString = copyString;
        function copyFloat(value) {
            if (typeof value !== "number") {
                throw new Error(`Expected number but received: ${value}`);
            }
            else if (!isFinite(value)) {
                throw new Error(`Expected finite number but received: ${value}`);
            }
            else if (value === -0) {
                return +0;
            }
            return value;
        }
        Validation.copyFloat = copyFloat;
        function copyInteger(value) {
            const result = copyFloat(value);
            if (result !== Math.floor(result)) {
                throw new Error(`Expected integer but received: ${value}`);
            }
            return result;
        }
        Validation.copyInteger = copyInteger;
        function copyBoolean(value) {
            if (typeof value !== "boolean") {
                throw new Error(`Expected boolean but received: ${value}`);
            }
            return value;
        }
        Validation.copyBoolean = copyBoolean;
        function copyUnsafeJSON(value) {
            if (value === null) {
                throw new Error(`Expected UnsafeJSON but received: ${value}`);
            }
            else if (typeof value !== "object") {
                throw new Error(`Expected UnsafeJSON but received: ${value}`);
            }
            else if (value.constructor !== Object) {
                throw new Error(`Expected plain object UnsafeJSON but received: ${value}`);
            }
            return value;
        }
        Validation.copyUnsafeJSON = copyUnsafeJSON;
        function copyOptional(wrappedValidator, value) {
            if (value === undefined || value === null)
                return undefined;
            return wrappedValidator(value);
        }
        Validation.copyOptional = copyOptional;
        function copyArray(elementValidator, value) {
            if (!Array.isArray(value)) {
                throw new Error(`Validation expected array but received: ${value}`);
            }
            return Object.freeze(value.map(element => elementValidator(element)));
        }
        Validation.copyArray = copyArray;
    })(Validation = ServiceRuntimePrivate.Validation || (ServiceRuntimePrivate.Validation = {}));
})(ServiceRuntimePrivate = exports.ServiceRuntimePrivate || (exports.ServiceRuntimePrivate = {}));
function assertNever(x, error) {
    throw error || new Error("Unexpected object: " + x);
}
exports.assertNever = assertNever;
