"use strict";
// ♻️
// IMPORTANT: Take care to apply changes to all supported languages
// when modifying code or documentation in the services runtime.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ServiceChannel_1 = require("./ServiceChannel");
const ServiceErrors_1 = require("./ServiceErrors");
const RuntimePrivate_1 = require("./RuntimePrivate");
const RuntimeDebugging_1 = require("./RuntimeDebugging");
const timers_1 = require("./timers");
/**
 * Manages a cluster of channels and services and sets up service communication. Use the auto-generated Service objects
 * to register implementations and get proxies to cross-process services. Most clients will set up a single shared object
 * with multiple channels.
 */
class ServiceManager {
    constructor(logger = RuntimeDebugging_1.ServiceDebugging.createLogger()) {
        this.logger = logger;
        /**
         * Creates a new proxy for a given service. Since the presence of the service isn't checked, errors may occur when
         * actually calling methods on the proxy.
         *
         * @param service The auto-generated definition object for the wanted service
         * @param channel Where the service is assumed to live. Calling this method implicitly registers the channel.
         */
        this.expect = (service, channel, options = {}) => {
            return this.addRouter(channel).expect(service, options);
        };
        /**
         * Waits for the presence of the service, with a configurable timeout. Returns a proxy on success, or rejects with
         * a service error (e.g. the request timed out, service wasn't).
         *
         * const service = await SomeService.on(someChannel).discover({ timeout: 1000 })
         * await service.someMethod()
         *
         * @param service The auto-generated definition object for the wanted service
         * @param channel Where the service is assumed to live. Calling this method implicitly registers the channel.
         * @param options.timeout Number of milliseconds to wait for the service's presence. Default = 30000
         */
        this.discover = (service, channel, options = {}) => __awaiter(this, void 0, void 0, function* () {
            return this.addRouter(channel).discover(service, options);
        });
        /** Removes a registered implementation or channel. */
        this.unregister = (what) => __awaiter(this, void 0, void 0, function* () {
            const routersForChannel = [];
            for (const router of this.routers) {
                if (router.channel === what) {
                    routersForChannel.push(router);
                }
            }
            if (routersForChannel.length > 0) {
                // Channel and its service implementations
                for (const router of routersForChannel) {
                    yield router.unregisterAllImplementations();
                    this.routers.delete(router);
                }
            }
            else {
                // Service implementation, or random value that will be ignored
                for (const router of this.routers) {
                    yield router.unregisterImplementation(what);
                }
            }
        });
        this.routers = new Set();
        this.addRouter = (channel) => {
            for (const existing of this.routers) {
                if (existing.channel === channel) {
                    return existing;
                }
            }
            const router = new Router(channel, this.logger);
            this.routers.add(router);
            return router;
        };
    }
    register(what) {
        return __awaiter(this, void 0, void 0, function* () {
            function isImplementation(obj) {
                return obj.service !== undefined && obj.implementation !== undefined;
            }
            if (isImplementation(what)) {
                yield this.addRouter(what.channel).registerImplementation(what.implementation, what.service);
            }
            else {
                this.addRouter(what.channel);
            }
        });
    }
}
exports.ServiceManager = ServiceManager;
(function (ServiceManager) {
    const sharedServiceManager = new ServiceManager();
    /**
     * Shared service manager for TypeScript clients that only have one area of responsibility within the runtime,
     * exposed as the generated `SomeService.on(channel)` convenience API. If your current script runtime only has
     * a single domain of responsibility (e.g. Vekter representing a single document), use the convenience instead
     * of constructing your own service managers.
     */
    function shared() {
        return RuntimeDebugging_1.ServiceDebugging._sharedServiceManagerIfTesting() || sharedServiceManager;
    }
    ServiceManager.shared = shared;
})(ServiceManager = exports.ServiceManager || (exports.ServiceManager = {}));
class Router {
    constructor(channel, logger) {
        this.channel = channel;
        this.logger = logger;
        this.onMessage = (message) => {
            try {
                // For now, just assume we're in a wonderful world where a type check implies a complete and valid message
                if (message.type === ServiceChannel_1.ServiceChannel.MessageType.Request) {
                    void this.onRequest(message);
                }
                else if (message.type === ServiceChannel_1.ServiceChannel.MessageType.Response) {
                    if (message.method === Discovery.method) {
                        this.onDiscoveryResponse(message);
                    }
                    else {
                        this.onResponse(message);
                    }
                }
                else if (message.type === ServiceChannel_1.ServiceChannel.MessageType.Error) {
                    void this.onError(message);
                }
                else {
                    RuntimePrivate_1.assertNever(message.type, new Error(`Unknown message: ${JSON.stringify(message)}`));
                }
            }
            catch (error) {
                this.logger.warn("☎️ onMessage", error);
            }
        };
        this.waitingDiscoveryMap = {};
        this.reflectDiscoveredServices = () => {
            if (!this.latestDiscoveryInfo) {
                return;
            }
            for (const serviceId of Object.keys(this.latestDiscoveryInfo.services)) {
                const promises = this.waitingDiscoveryMap[serviceId];
                if (promises) {
                    this.waitingDiscoveryMap[serviceId] = [];
                    promises.forEach(promise => promise.resolve());
                }
            }
        };
        this.onDiscoveryResponse = (response) => {
            if (Discovery.isValidInfo(response.body)) {
                this.latestDiscoveryInfo = response.body;
                this.reflectDiscoveredServices();
                this.logger.log(`↘️ Discovered services`, response.body);
            }
            else {
                this.logger.warn("☎️ onDiscoveryResponse: invalid data", response.body);
            }
            // Also do standard response handling if it's an explicit discovery request
            if (response.id !== Discovery.broadcastMessageId) {
                this.onResponse(response);
            }
        };
        this.broadcastDiscoveryInfo = (requestId) => {
            const services = {};
            for (const key in this.implementedServices) {
                const service = this.implementedServices[key].service;
                services[key] = { fingerprint: service.fingerprint };
            }
            const discoveryInfo = { services };
            try {
                this.channel.postMessage({
                    type: ServiceChannel_1.ServiceChannel.MessageType.Response,
                    id: requestId || Discovery.broadcastMessageId,
                    serviceId: Discovery.serviceId,
                    method: Discovery.method,
                    body: discoveryInfo,
                });
            }
            catch (_a) {
                // postMessage can fail (e.g. when generating a "dehydrated" version of the host page). Since this broadcast happens automatically and not as part of explicitly calling a service method, suppress errors here. If discovery fails, the actual services will fail as well.
            }
        };
        // Outgoing messages
        this.expect = (service, options) => {
            this.logger.log(`☎️ expect ${service.id}`);
            return service.newOutgoingWrapper(options, (message) => __awaiter(this, void 0, void 0, function* () {
                // Discover services if needed
                yield this.waitForDiscoveryInfo();
                this.throwErrorIfBadService(service);
                return this.postRequest(service.id, message);
            }));
        };
        this.discover = (service, _a = {}) => __awaiter(this, void 0, void 0, function* () {
            var { timeout = 30000 } = _a, options = __rest(_a, ["timeout"]);
            this.logger.log(`☎️ discover ${service.id}`);
            // Discover services if needed and wait until this specific one appears
            yield Promise.race([
                this.waitForDiscoveredService(service),
                // Time out if it doesn't appear fast enough
                new Promise((_, reject) => timers_1.setTimeout(() => reject(
                // Distinguish between timeout waiting for discovery and waiting for a service to appear
                this.latestDiscoveryInfo ? new ServiceErrors_1.ServiceError.ServiceNotFound() : new ServiceErrors_1.ServiceError.TimedOut()), timeout)),
            ]);
            this.throwErrorIfBadService(service);
            return this.expect(service, options);
        });
        this.throwErrorIfBadService = (service) => {
            const discoveryInfo = this.latestDiscoveryInfo;
            // Fail if the other end of the channel doesn't declare the needed service
            const serviceInfo = discoveryInfo && discoveryInfo ? discoveryInfo.services[service.id] : undefined;
            if (!serviceInfo) {
                this.logger.log(`☎️ Couldn't find service ${service.id}`, discoveryInfo);
                throw new ServiceErrors_1.ServiceError.ServiceNotFound();
            }
            // Fail if the versions don't match
            if (serviceInfo.fingerprint !== service.fingerprint) {
                this.logger.log(`☎️ Couldn't find service with required version fingerprint. Make sure both endpoints are using the same version of the Services package.`, discoveryInfo);
                throw new ServiceErrors_1.ServiceError.ServiceRequiresUpgrade();
            }
        };
        this.postRequest = (serviceId, message, timeout) => {
            this.logger.log(`↗️ ${serviceId}`, message);
            const responsePromise = RuntimePrivate_1.ServiceRuntimePrivate.newResolvablePromise();
            // Prepare a good enough unique string to identify the request/response pair across services
            const requestId = RuntimePrivate_1.ServiceRuntimePrivate.generateUniqueId();
            this.waitingRequestsMap[requestId] = responsePromise;
            // Send the request
            const request = {
                type: ServiceChannel_1.ServiceChannel.MessageType.Request,
                id: requestId,
                serviceId,
                method: message.method,
                stream: StreamOperation.toMessage(message.stream),
                body: message.argument,
            };
            this.channel.postMessage(request);
            // Simulate an error if there has been no response after the timeout
            if (typeof timeout === "number") {
                timers_1.setTimeout(() => {
                    const needsResponse = !!this.waitingRequestsMap[requestId];
                    if (needsResponse) {
                        this.onError({
                            type: ServiceChannel_1.ServiceChannel.MessageType.Error,
                            id: request.id,
                            serviceId: request.serviceId,
                            method: request.method,
                            body: undefined,
                        }, new ServiceErrors_1.ServiceError.TimedOut());
                    }
                }, timeout);
            }
            // Get the regular result, then optionally validate it
            return responsePromise.then(body => {
                if (body && message.returnValidator) {
                    try {
                        return message.returnValidator(body);
                    }
                    catch (error) {
                        const typeContext = RuntimePrivate_1.ServiceRuntimePrivate.Validation.typeContextForError(error);
                        throw new ServiceErrors_1.ServiceError.BadResponse((typeContext ? `${typeContext}: ` : "") + error.message);
                    }
                }
                return body;
            });
        };
        this.waitingRequestsMap = {};
        this.onResponse = (response) => {
            const request = this.waitingRequestsMap[response.id];
            if (!request)
                return this.logger.warn("☎️ onResponse: couldn't find request", response);
            delete this.waitingRequestsMap[response.id];
            request.resolve(response.body);
        };
        this.onError = (response, customError) => {
            const request = this.waitingRequestsMap[response.id];
            if (!request)
                return this.logger.warn("☎️ onError: couldn't find request", response);
            delete this.waitingRequestsMap[response.id];
            const error = customError || ServiceErrors_1.ServiceError.fromMessageBody(response.body);
            request.reject(error);
        };
        // Incoming messages
        this.implementedServices = {};
        this.registerImplementation = (rawImplementation, service) => __awaiter(this, void 0, void 0, function* () {
            this.logger.log("☎️ registerImplementation", rawImplementation);
            // Only expose known methods, bound to the implementation object so they can be called as is
            const implementation = {};
            for (const key in service.methods) {
                const name = key;
                const method = rawImplementation[name];
                if (typeof method !== "function") {
                    throw new ServiceErrors_1.ServiceError.Implementation(`Implementation for ${service.id} doesn't correctly implement ${name}()`);
                }
                implementation[name] = method.bind(rawImplementation);
            }
            this.implementedServices[service.id] = {
                service,
                rawImplementation,
                implementation: Object.freeze(implementation),
            };
            this.broadcastDiscoveryInfo();
        });
        this.unregisterImplementation = (rawImplementation) => __awaiter(this, void 0, void 0, function* () {
            const shouldUnregisterAll = rawImplementation === this.unregisterAllToken;
            if (!shouldUnregisterAll)
                this.logger.log("☎️ unregisterImplementation", rawImplementation);
            // ServiceManager unregisters the implementation on all routers, regardless of whether it was registered at all
            let didChangeServices = false;
            const streamIdsToCancel = [];
            for (const serviceId of Object.keys(this.implementedServices)) {
                if (!shouldUnregisterAll && this.implementedServices[serviceId].rawImplementation !== rawImplementation) {
                    continue;
                }
                // Delete the service
                delete this.implementedServices[serviceId];
                didChangeServices = true;
                // Find associated streams
                for (const streamId of Object.keys(this.requestedStreamsMap)) {
                    if (this.requestedStreamsMap[streamId].serviceId === serviceId) {
                        streamIdsToCancel.push(streamId);
                    }
                }
            }
            yield this.cancelStreams(streamIdsToCancel, new ServiceErrors_1.ServiceError.ServiceGone());
            if (didChangeServices) {
                this.broadcastDiscoveryInfo();
            }
        });
        this.unregisterAllToken = {};
        this.unregisterAllImplementations = () => __awaiter(this, void 0, void 0, function* () {
            this.logger.log("☎️ unregisterAllImplementations");
            yield this.unregisterImplementation(this.unregisterAllToken);
        });
        this.requestedStreamsMap = {};
        this.cancelStreams = (streamIds, error) => __awaiter(this, void 0, void 0, function* () {
            const cancelPromises = [];
            for (const streamId of streamIds) {
                const stream = this.requestedStreamsMap[streamId];
                delete this.requestedStreamsMap[streamId];
                // Stop the iteration
                if (stream && stream.iterator.throw) {
                    cancelPromises.push(stream.iterator.throw(error));
                }
            }
            yield Promise.all(cancelPromises);
        });
        this.onRequest = (request) => __awaiter(this, void 0, void 0, function* () {
            // Explicit discovery requests
            if (request.method === Discovery.method) {
                this.broadcastDiscoveryInfo(request.id);
                return;
            }
            // Service requests
            let resultType = ServiceChannel_1.ServiceChannel.MessageType.Response;
            let resultBody;
            try {
                const implementedService = this.implementedServices[request.serviceId];
                const implementation = implementedService ? implementedService.implementation : undefined;
                if (!implementation) {
                    throw new ServiceErrors_1.ServiceError.BadRequest();
                }
                this.logger.log(`↘️ ${request.serviceId}`, request);
                if (request.stream) {
                    // Streaming method, a.k.a. an iteration request for an async iterator
                    const { id: streamId, method: streamMethod } = StreamOperation.fromMessage(request.stream);
                    let stream = this.requestedStreamsMap[streamId];
                    if (!stream && streamMethod === "next") {
                        // Start a new iterator if one doesn't exist yet
                        const method = implementation[request.method];
                        const iteratorStream = (yield method(request.body));
                        const iteratorOptions = RuntimePrivate_1.ServiceRuntimePrivate.isServiceEventStreamOptions(request.body)
                            ? request.body
                            : undefined;
                        stream = {
                            iterator: iteratorStream[Symbol.asyncIterator](iteratorOptions),
                            serviceId: implementedService.service.id,
                        };
                        this.requestedStreamsMap[streamId] = stream;
                    }
                    if (streamMethod === "next") {
                        // Get the next iteration
                        const iteratorResult = yield stream.iterator.next();
                        resultBody = { done: iteratorResult.done, value: iteratorResult.value };
                    }
                    else if (streamMethod === "return") {
                        // Stop the iteration
                        if (stream.iterator.return) {
                            yield stream.iterator.return();
                        }
                        // Remove the stream
                        delete this.requestedStreamsMap[streamId];
                        resultBody = { done: true, value: undefined };
                    }
                    else {
                        throw new ServiceErrors_1.ServiceError.BadRequest("Stream operations other than next() and return() are not yet supported");
                    }
                }
                else {
                    // Regular service method
                    const method = implementation[request.method];
                    resultBody = yield method(request.body);
                }
            }
            catch (error) {
                this.logger.log("☎️ onRequest: error", request, error);
                resultType = ServiceChannel_1.ServiceChannel.MessageType.Error;
                resultBody = ServiceErrors_1.ServiceError.toMessageBody(error);
            }
            finally {
                // Note: the result type currently isn't verified or validated in any way
                this.channel.postMessage({
                    type: resultType,
                    id: request.id,
                    serviceId: request.serviceId,
                    method: request.method,
                    body: resultBody,
                });
            }
        });
        channel.addMessageListener(this.onMessage);
    }
    waitForDiscoveryInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            // No need to discover if we already have info. The other end of the channel makes sure that any updates get
            // actively pushed using broadcastDiscoveryInfo (e.g. when a service registers)
            if (this.latestDiscoveryInfo) {
                return this.latestDiscoveryInfo;
            }
            // Explicitly request discovery info
            yield this.postRequest(Discovery.serviceId, { method: Discovery.method }, 1000 /* timeout */);
            if (!this.latestDiscoveryInfo) {
                throw new ServiceErrors_1.ServiceError.ServiceNotFound();
            }
            return this.latestDiscoveryInfo;
        });
    }
    waitForDiscoveredService(service) {
        return __awaiter(this, void 0, void 0, function* () {
            // Will be resolved when any discovery response contains the expected service
            const promise = RuntimePrivate_1.ServiceRuntimePrivate.newResolvablePromise();
            const promises = this.waitingDiscoveryMap[service.id] || [];
            this.waitingDiscoveryMap[service.id] = promises;
            promises.push(promise);
            // Make sure there's at least one attempt to get discovery info, but ignore its success/failure
            this.waitForDiscoveryInfo().catch(() => { });
            this.reflectDiscoveredServices();
            return promise;
        });
    }
}
var Discovery;
(function (Discovery) {
    Discovery.serviceId = "";
    Discovery.method = "#discover";
    Discovery.broadcastMessageId = "";
    function isValidInfo(body) {
        if (!body || !body.services || typeof body.services !== "object")
            return false;
        return true;
    }
    Discovery.isValidInfo = isValidInfo;
})(Discovery || (Discovery = {}));
var StreamOperation;
(function (StreamOperation) {
    const returnPrefix = "#return:";
    const throwPrefix = "#throw:";
    function fromMessage(operation) {
        if (operation.startsWith(returnPrefix)) {
            return { id: operation.substr(returnPrefix.length), method: "return" };
        }
        else if (operation.startsWith(throwPrefix)) {
            return { id: operation.substr(throwPrefix.length), method: "throw" };
        }
        else {
            return { id: operation, method: "next" };
        }
    }
    StreamOperation.fromMessage = fromMessage;
    function toMessage(stream) {
        if (!stream)
            return undefined;
        switch (stream.method) {
            case "next":
                return stream.id; // Having just a stream identifier implicitly means calling next()
            case "return":
                return returnPrefix + stream.id;
            case "throw":
                return throwPrefix + stream.id;
            default:
                return undefined;
        }
    }
    StreamOperation.toMessage = toMessage;
})(StreamOperation || (StreamOperation = {}));
