"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const RuntimePrivate_1 = require("./RuntimePrivate");
const ServiceErrors_1 = require("./ServiceErrors");
const RuntimeDebugging_1 = require("./RuntimeDebugging");
const logger = RuntimeDebugging_1.ServiceDebugging.createLogger();
/**
 * ServiceEventEmitter is a stream implementation for a (potentially infinite) sequence of events
 * not associated with any particular task or service message. It supports any number of attached
 * AsyncIterators, with no guarantee as to which event will appear in the first iteration. Use for
 * publish/subscribe or state observing behavior.
 */
class ServiceEventEmitter {
    constructor() {
        this.iterators = [];
        this.emit = (value) => {
            this.latestValue = value;
            for (const iterator of this.iterators) {
                iterator.update(value);
            }
        };
        this.read = (_) => __awaiter(this, void 0, void 0, function* () {
            // Conform to the client+server ServiceStream interface, but make sure the runtime never depends on it
            throw new ServiceErrors_1.ServiceError.BadRequest("read() on ServiceEventEmitter implementations is invalid");
        });
        this.cancel = () => __awaiter(this, void 0, void 0, function* () {
            throw new ServiceErrors_1.ServiceError.BadRequest("cancel() on ServiceEventEmitter implementations is invalid");
        });
    }
    [Symbol.asyncIterator](options) {
        const id = RuntimePrivate_1.ServiceRuntimePrivate.generateUniqueId();
        return new ServiceTask(
        // Individual stream
        (update, done) => {
            this.iterators.push({ id, update, done });
            // Some streams may want an initial value instead of waiting for a new emit
            if (options && options.replay === "latest") {
                if (this.latestValue) {
                    update(this.latestValue);
                }
                else {
                    // Since value replay is opt-in for service definitions, enforce correct implementation
                    throw new ServiceErrors_1.ServiceError.Implementation(`ServiceEventEmitter received a new stream requesting the last value but no value has been emitted`);
                }
            }
        }, 
        // Ended iteration (either from client or server side)
        () => {
            const i = this.iterators.findIndex(iterator => iterator.id === id);
            if (i >= 0) {
                this.iterators.splice(i, 1);
            }
            else {
                throw new ServiceErrors_1.ServiceError.BadRequest(`ServiceEventEmitter couldn't find cancelled iterator with id: ${id}`);
            }
        });
    }
}
exports.ServiceEventEmitter = ServiceEventEmitter;
/**
 * ServiceTask is a stream implementation for a finite sequence of updates or data packets, sent as a result of
 * one specific service message. It can be iterated only once. Use for one-off task status updates or data transfer.
 */
class ServiceTask {
    constructor(task, onIteratorEnd) {
        this.onIteratorEnd = onIteratorEnd;
        this.hasAsyncIterator = false;
        this.promises = [];
        this.doneResult = { done: true, value: undefined };
        this.update = (value) => {
            let lastPromise = this.promises[this.promises.length - 1];
            if (value && lastPromise === undefined) {
                if (!this.returnedNextPromise) {
                    logger.warn("lastPromise and returnedNextPromise should never both be undefined");
                    return;
                }
                lastPromise = this.returnedNextPromise;
            }
            if (value === undefined) {
                // Done: clean up the last next() if necessary (may already have happened if cancel/return calls this)
                lastPromise && lastPromise.resolve(this.doneResult);
            }
            else if (value instanceof ServiceErrors_1.ServiceError) {
                // Error: clean up the last next() if necessary (may already have happened if cancel/return calls this)
                lastPromise && lastPromise.reject(value);
            }
            else {
                // Update: always prepare a promise for the next next()
                this.promises.push(RuntimePrivate_1.ServiceRuntimePrivate.newResolvablePromise());
                lastPromise && lastPromise.resolve({ done: false, value });
            }
        };
        this.next = () => __awaiter(this, void 0, void 0, function* () {
            const promise = this.promises.shift();
            this.returnedNextPromise = promise;
            return promise || this.doneResult;
        });
        this.return = () => __awaiter(this, void 0, void 0, function* () {
            this.update(undefined);
            this.onIteratorEnd && this.onIteratorEnd();
            return this.doneResult;
        });
        this.throw = (error) => __awaiter(this, void 0, void 0, function* () {
            this.update(error);
            this.onIteratorEnd && this.onIteratorEnd();
            return this.doneResult;
        });
        this.read = (_) => __awaiter(this, void 0, void 0, function* () {
            // Conform to the client+server ServiceStream interface, but make sure the runtime never depends on it
            throw new ServiceErrors_1.ServiceError.BadRequest("read() on ServiceTask implementations is invalid");
        });
        this.promises = [RuntimePrivate_1.ServiceRuntimePrivate.newResolvablePromise()];
        task(this.update, this.update);
    }
    [Symbol.asyncIterator]() {
        if (this.hasAsyncIterator) {
            throw new Error("ServiceTask.asyncIterator() may only be called once");
        }
        this.hasAsyncIterator = true;
        return this;
    }
}
exports.ServiceTask = ServiceTask;
