"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const resolve_pkg_1 = __importDefault(require("resolve-pkg"));
const ini_1 = __importDefault(require("ini"));
const logger_1 = require("./logger");
const logger = logger_1.createLogger();
const nodePath = process.execPath;
// 1. Find yarn dependency path in a parent directory
// 2. Follow the bin.yarn path from the package.json field
const yarnPackagePath = resolve_pkg_1.default("yarn", { cwd: __dirname });
if (!yarnPackagePath) {
    throw new Error("Could not find Yarn executable.");
}
exports.yarnPath = path_1.default.resolve(yarnPackagePath, JSON.parse(fs_extra_1.default.readFileSync(path_1.default.join(yarnPackagePath, "package.json"), "utf8")).bin.yarn);
function log(result) {
    if (result.output) {
        logger.log(result.output);
    }
    if (result.error) {
        logger.error(result.error);
    }
    return result;
}
class Yarn {
    constructor(initialization = Promise.resolve(), token) {
        this.initialization = initialization;
        this.run = (args, callback) => __awaiter(this, void 0, void 0, function* () {
            yield this.initialization;
            return new Promise((resolve, reject) => {
                args.unshift(exports.yarnPath);
                const output = [];
                const errorBuffers = [];
                const childProcess = child_process_1.spawn(nodePath, args, {
                    cwd: process.cwd(),
                    env: Object.assign(Object.assign({}, process.env), { FRAMER_TOKEN: this.token }),
                });
                const outputHandler = jsonLines((line) => {
                    if (typeof line === "string") {
                        // Line that could not be parsed as JSON.
                        output.push(line + "\n");
                        return;
                    }
                    switch (line.type) {
                        case "progressFinish":
                        case "progressStart":
                        case "progressTick":
                            // TODO: Add support for reporting progress. Ignore for now.
                            break;
                        case "info":
                        case "step":
                            // TODO: Decide if we want to be more verbose about these messages.
                            break;
                        case "tree":
                        // TODO: Stop passing through success as JSON (we rely on this behavior for now):
                        // case "success":
                        default:
                            // TODO: Handle most messages and stop outputting unhandled ones.
                            output.push(JSON.stringify(line) + "\n");
                            break;
                    }
                });
                const errorHandler = jsonLines((line) => {
                    if (typeof line === "string" || line.type !== "error") {
                        return;
                    }
                    logger.error(`yarn ${args[1]}: ${line.data}`);
                });
                childProcess.stdout.on("data", outputHandler);
                childProcess.stderr.on("data", (chunk) => {
                    errorHandler(chunk);
                    errorBuffers.push(chunk);
                });
                const result = () => {
                    return { output: output.join(""), error: Buffer.concat(errorBuffers).toString() };
                };
                childProcess.on("error", () => reject(log(result())));
                childProcess.on("close", () => setTimeout(() => resolve(log(result())), 100));
            }).then(result => {
                if (callback) {
                    callback();
                }
                return result;
            });
        });
        this.install = args => {
            return this.run(["install", ...args]);
        };
        this.link = args => {
            return this.run(["link", ...args]);
        };
        this.unlink = args => {
            return this.run(["unlink", ...args]);
        };
        this.add = args => {
            return this.run(["add", ...args]);
        };
        this.remove = args => {
            return this.run(["remove", ...args]);
        };
        this.publish = args => {
            return this.run(["publish", ...args]);
        };
        this.outdated = args => {
            return this.run(["outdated", ...args]);
        };
        this.upgrade = args => {
            return this.run(["upgrade", ...args]);
        };
        this.check = args => {
            return this.run(["check", ...args]);
        };
        this.info = args => {
            return this.run(["info", ...args]);
        };
        this.token = token;
    }
}
exports.Yarn = Yarn;
/**
 * Takes Buffer chunks and parses each complete line as a JSON object and passes it on to the
 * provided handler. Lines that could not be parsed as JSON will be passed through as strings.
 */
function jsonLines(handler) {
    let buffer = "";
    return (chunk) => {
        const lines = (buffer + chunk.toString()).split("\n");
        if (lines.length === 1) {
            // No newline in chunk, so it's probably incomplete.
            buffer = lines[0];
            return;
        }
        buffer = lines.pop() || ""; // Non-empty if chunk did not end with newline.
        for (const line of lines) {
            if (!line)
                continue;
            try {
                const message = JSON.parse(line);
                handler(message);
            }
            catch (e) {
                handler(line);
            }
        }
    };
}
// Handling writing npmrc for use by Yarn for Framer registry access
function writeNpmrc(projectPath, userInfo) {
    return __awaiter(this, void 0, void 0, function* () {
        const npmrcPath = path_1.default.join(projectPath, ".npmrc");
        const hasNpmrc = yield fs_extra_1.default.pathExists(npmrcPath);
        if (!userInfo) {
            if (!hasNpmrc) {
                logger.error("Cannot fetch or find .npmrc file in project.");
            }
            return;
        }
        // Default to writing the npmrc content that is returned from the API
        let { npmrc } = userInfo;
        // If there is an existing npmrc file, we need to merge the contents to
        // protect the user's potential settings for their own registries
        if (hasNpmrc) {
            // Parse the existing npmrc via ini and turn back into a string
            // Merge the template npmrc with the existing npmrc content
            // Any of the new values that have matching keys will be preferred over
            // existing values that have the same key
            // This will resolve keys changing and help future-proof against
            // new key/values that might come from the API's npmrc template in the
            // future
            const existingNpmrc = yield fs_extra_1.default.readFile(npmrcPath, "utf8");
            const content = [existingNpmrc, npmrc].join("\n");
            npmrc = ini_1.default.stringify(ini_1.default.parse(content));
        }
        return fs_extra_1.default.outputFile(npmrcPath, npmrc, "utf8");
    });
}
exports.writeNpmrc = writeNpmrc;
// We need to write this to the yarnrc because it is not picked up when put in the npmrc
// 1. The link-folder command line argument is also passed by the server, this is only so that external yarn commands also pick the right link folder
//    Otherwise running `yarn add` in the project folder would remove the linked framer library
// 2. The disable-self-update-check makes sure yarn does not prompt the user to update itself. As Framer comes with its
//    own version of yarn, this is not in the user's control.
// 3. The yarn-path argument is special - it gives a check for the proper configuration before executing the yarn command.
//    It also ensures that the yarn commands are being routed through our bundled version of the package rather than the clients.
function writeYarnrc(projectPath, yarnLinkDir) {
    const yarnrcPath = path_1.default.join(projectPath, ".yarnrc");
    const yarnrcContents = `
--link-folder "${yarnLinkDir}"
disable-self-update-check true
yarn-path "${path_1.default.resolve(__dirname, "framerYarn.js")}"
`;
    fs_extra_1.default.outputFileSync(yarnrcPath, yarnrcContents, "utf8");
}
exports.writeYarnrc = writeYarnrc;
