"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const assert_1 = __importDefault(require("assert"));
const lockfile_1 = __importDefault(require("@yarnpkg/lockfile"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const resolve_pkg_1 = __importDefault(require("resolve-pkg"));
const Logger_1 = require("../Logger");
const fs_1 = require("../fs");
const utils_1 = require("./utils");
const config_1 = require("../config");
const logger = new Logger_1.Logger("framer-build-library:library-manager");
class FramerLibraryManager {
    constructor({ paths, pkg, builtInFramerVersion = config_1.DEFAULT_BUILT_IN_FRAMER_VERSION }) {
        this.pkg = pkg;
        this.paths = paths;
        this.builtInFramerVersion = builtInFramerVersion;
    }
    /**
     * @returns true if actions were performed to change the installed library
     */
    async install(version) {
        const installedFramerVersion = await this.installedFramerVersion();
        const dependencies = await this.pkg.readDependencies();
        const currentInstallStrategy = await FramerLibraryManager.installStrategy(dependencies);
        let updatedInstallStrategy;
        if (version) {
            updatedInstallStrategy = FramerLibraryManager.installStrategyForVersion(version, this.builtInFramerVersion);
        }
        else {
            updatedInstallStrategy = FramerLibraryManager.updateInstallStrategy(currentInstallStrategy, this.builtInFramerVersion, installedFramerVersion);
        }
        if (!updatedInstallStrategy) {
            // We’re stuck :( This will only happen if all the info above is undefined
            throw Error("Can’t install Framer Library");
        }
        const script = FramerLibraryManager.generateActions(currentInstallStrategy, updatedInstallStrategy, installedFramerVersion === undefined ? null : installedFramerVersion);
        for (const action of script) {
            if (action === "remove") {
                await this.pkg.removeDependency("framer");
                continue;
            }
            if (action === "unlink") {
                await this.pkg.unlinkDependency("framer");
                continue;
            }
            if (action === "link") {
                await this.pkg.linkDependency("framer");
                continue;
            }
            if (action.name === "add") {
                await this.pkg.addDependency("framer", action.version);
                continue;
            }
            if (action.name === "addDev") {
                await this.pkg.addDevDependency("framer", action.version);
                continue;
            }
            if (action.name === "addPeer") {
                const { framer: currentFramerPeerDependency } = dependencies.peerDependencies;
                const peerDependencyVersion = FramerLibraryManager.newPeerDependency(currentFramerPeerDependency, action.version);
                await this.pkg.addPeerDependency("framer", peerDependencyVersion);
                continue;
            }
            if (action.name === "upgrade") {
                await this.pkg.upgradeDependency("framer", action.version);
                continue;
            }
            // Make sure we’re exhaustive
            assert_1.default.ok(!action.name, `Unexpected value: ${action.name || action}`);
        }
        // Re-fetch the library version post actions.
        const currentFramerVersion = await this.installedFramerVersion();
        if (currentFramerVersion === undefined) {
            logger.warn("Was not able to install Framer version");
        }
        // We could do an early return, but then we would not update the current library version
        return script.length > 0;
    }
    getPublishedFramerVersions() {
        return this.pkg.info("framer", "versions");
    }
    getPackageJson(packagePath) {
        return fs_1.readJSON(path_1.default.join(packagePath, "package.json"));
    }
    async getPackageVersion(packagePath) {
        const packageJson = await this.getPackageJson(packagePath);
        return (packageJson && packageJson.version) || undefined;
    }
    async installedFramerVersion() {
        // Try and resolve the version from an installed package.
        const version = await this.resolvePackageFramerVersion();
        if (version) {
            return version;
        }
        const installation = FramerLibraryManager.installStrategy(await this.pkg.readDependencies());
        if (installation) {
            try {
                const yarnLockfile = lockfile_1.default.parse((await fs_extra_1.default.readFile(this.paths.yarnLock)).toString());
                const lockedFramer = yarnLockfile.object[`framer@${installation.version}`];
                if (lockedFramer) {
                    return lockedFramer.version;
                }
            }
            catch (error) {
                // Fall through, we’ll handle errors once
            }
        }
    }
    async libraryCompatibility() {
        return utils_1.libraryCompatibility(await this.pkg.readDependencies(), this.builtInFramerVersion, await this.installedFramerVersion());
    }
    // Find the version of the package's installed framer dependency (if it exists)
    async resolvePackageFramerVersion() {
        // Search for the Framer module, understanding that hoisting can make it
        // so the package is installed in a parent directory's node_modules
        const framerPath = await resolve_pkg_1.default("framer", { cwd: this.paths.project });
        let framerPathStats;
        if (!framerPath) {
            return;
        }
        try {
            framerPathStats = await fs_extra_1.default.lstat(framerPath);
        }
        catch (error) {
            return;
        }
        // We need to verify that the installed package is still valid for
        // the current filesystem and Framer version. It must either be
        // a directory or symlink to the current yarn link-folder.
        if (framerPathStats.isSymbolicLink()) {
            const symlinkPath = await fs_extra_1.default.readlink(framerPath);
            const resolvedPath = path_1.default.resolve(path_1.default.dirname(framerPath), symlinkPath);
            if (this.paths.yarnLinkDir && !resolvedPath.startsWith(this.paths.yarnLinkDir)) {
                return;
            }
        }
        else if (!framerPathStats.isDirectory()) {
            return;
        }
        // Try and get the current version from the installed package.
        return this.getPackageVersion(framerPath);
    }
}
exports.FramerLibraryManager = FramerLibraryManager;
FramerLibraryManager.updateInstallStrategy = utils_1.updateInstallStrategy;
FramerLibraryManager.installStrategy = utils_1.installStrategy;
FramerLibraryManager.generateActions = utils_1.generateActions;
FramerLibraryManager.latestOfMajorVersions = utils_1.latestOfMajorVersions;
FramerLibraryManager.newPeerDependency = utils_1.newPeerDependency;
FramerLibraryManager.isCompatible = utils_1.isCompatible;
FramerLibraryManager.installStrategyForVersion = utils_1.installStrategyForVersion;
