"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const assert_1 = __importDefault(require("assert"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const v4_1 = __importDefault(require("uuid/v4"));
const config_1 = require("../config");
const Compiler_1 = require("../Compiler");
const Package_1 = require("../Package/Package");
const Logger_1 = require("../Logger");
const Paths_1 = require("./Paths");
exports.ProjectPaths = Paths_1.ProjectPaths;
const logger = new Logger_1.Logger("framer-build-library:project");
class Project {
    constructor({ paths, pkg, compiler, env = {}, }) {
        this.paths = paths;
        this.path = this.paths.project;
        this.pkg = pkg;
        this.compiler = compiler;
        this.pkg.setYarnEnv(env);
    }
    static async load(config) {
        const { cachePath, yarnLinkDir, npmrcTemplate, env, builtInFramerVersion = config_1.DEFAULT_BUILT_IN_FRAMER_VERSION, } = config;
        const projectPath = await Project.prepareDir({ directory: config.projectPath });
        const paths = new Paths_1.ProjectPaths({
            projectPath,
            cachePath,
            yarnLinkDir,
        });
        const pkg = await Package_1.Package.load({ npmrcTemplate, paths, builtInFramerVersion });
        const compiler = new Compiler_1.Compiler({
            ...config.compiler,
            root: paths.project,
        });
        const project = new Project({ paths, pkg, compiler, env });
        await project.install();
        return project;
    }
    static async create(config) {
        const { project: projectPath } = new Paths_1.ProjectPaths(config);
        const templatePath = await Project.extract(path_1.default.resolve(config.templatePath));
        if (await fs_extra_1.default.pathExists(projectPath)) {
            throw new Error(`${projectPath} already exists and is being overwritten.`);
        }
        {
            const nodeModules = path_1.default.join(templatePath, "node_modules");
            const backups = path_1.default.join(templatePath, ".backups");
            await fs_extra_1.default.copy(templatePath, projectPath, {
                filter: file => !file.startsWith(nodeModules) && !file.startsWith(backups),
            });
        }
        return projectPath;
    }
    static async prepareDir({ directory }) {
        const projectPath = await Project.extract(directory);
        const paths = new Paths_1.ProjectPaths({
            projectPath,
        });
        await Promise.all([
            fs_extra_1.default.ensureDir(paths.buildOutputDir),
            fs_extra_1.default.ensureDir(paths.code),
            fs_extra_1.default.ensureDir(paths.design),
            fs_extra_1.default.ensureDir(paths.distOutputDir),
            fs_extra_1.default.ensureDir(paths.metadata),
        ]);
        return projectPath;
    }
    static extract(projectPath) {
        let newProjectPath = projectPath;
        if (projectPath.endsWith(".framerx")) {
            // This is a compressed archive.
            const archivePath = projectPath;
            newProjectPath = path_1.default.join(os_1.default.tmpdir(), v4_1.default(), "container");
            // TODO: Consider using blacklist to ignore unnecessary stuff (like .backups).
            return new Promise((resolve, reject) => {
                // eslint-disable-next-line @typescript-eslint/no-var-requires
                const extract = require("extract-zip");
                extract(archivePath, { dir: newProjectPath }, (error) => {
                    if (error) {
                        reject(error);
                        return;
                    }
                    resolve(newProjectPath);
                });
            });
        }
        return validateIsFramerProject(newProjectPath);
    }
    async install() {
        // Install dependencies, prefering already installed
        return this.pkg.install("--prefer-offline");
    }
    installFramer(version) {
        return this.pkg.installFramer(version);
    }
    addDependency(dependency, version) {
        return this.pkg.addDependency(dependency, version);
    }
    addPeerDependency(dependency, version) {
        return this.pkg.addPeerDependency(dependency, version);
    }
    addDevDependency(dependency, version) {
        return this.pkg.addDevDependency(dependency, version);
    }
    removeDependency(dependency) {
        return this.pkg.removeDependency(dependency);
    }
    getLatestPublishedVersion() {
        return this.pkg.getLatestPublishedVersion();
    }
    build() {
        return this.compiler.build();
    }
    async buildDist() {
        // await this.prepareForPublish(packageName, displayName)
        await this.install();
        const externals = Object.keys((await this.pkg.readDependencies(true)).dependencies);
        const compiler = new Compiler_1.Compiler({ root: this.paths.project, enableBundleSplit: false, externals });
        const build = await compiler.build();
        await this.writeBuildOutputToDisk(build);
        return build;
    }
    async publish({ version, packageName, displayName, access, }) {
        const options = ["--no-git-tag-version"];
        if (version) {
            options.push("--new-version", version);
        }
        if (access === "restricted" || access === "public") {
            options.push("--access", access);
        }
        if (packageName) {
            await this.setPackageName(packageName);
        }
        if (displayName) {
            await this.setDisplayName(displayName);
        }
        await this.buildDist();
        return this.pkg.publish(...options);
    }
    getDisplayName() {
        return this.pkg.readDisplayName();
    }
    getAuthor() {
        return this.pkg.readAuthor();
    }
    getPackageName() {
        return this.pkg.readPackageName();
    }
    getPackageVersion() {
        return this.pkg.readPackageVersion();
    }
    async getDependencies() {
        const { dependencies = {}, peerDependencies = {}, devDependencies = {} } = await this.pkg.readDependencies();
        return {
            dependencies,
            peerDependencies,
            devDependencies,
        };
    }
    async getPackages(...args) {
        const { type, tree } = await this.pkg.list(...args);
        const listTreeOutput = tree;
        if (type !== "success") {
            logger.error("Failed to list installed packages");
            return [];
        }
        if (!listTreeOutput[0] || !listTreeOutput[0].data || !listTreeOutput[0].data.trees) {
            logger.warn("Found no installed packages");
            return [];
        }
        return listTreeOutput[0].data.trees;
    }
    setDisplayName(name) {
        return this.pkg.writeDisplayName(name);
    }
    setAuthor(author) {
        return this.pkg.writeAuthor(author);
    }
    setPackageName(name) {
        return this.pkg.writePackageName(name);
    }
    setPackageVersion(version) {
        return this.pkg.writePackageVersion(version);
    }
    // Writing the in memory output from Webpack to the FS
    // requires injecting the canvas data into the index.js file
    // and writing each chunk to disk
    async writeBuildOutputToDisk(build) {
        if (!build) {
            logger.error("Attempted to write non-existing build to disk");
            return;
        }
        await Promise.all(Object.values(Compiler_1.BundleChunkName).map(fileName => {
            if (!build.chunks[fileName]) {
                return;
            }
            let fileContent = build.chunks[fileName].content();
            if (!fileContent) {
                logger.error(`There is no ${fileName} content in the build`);
                return;
            }
            if (fileName === Compiler_1.BundleChunkName.IndexJs) {
                fileContent = injectCanvasData(fileContent);
            }
            return fs_extra_1.default.writeFile(path_1.default.join(this.paths.distOutputDir, fileName), fileContent);
        })).catch(error => {
            logger.error(error);
            throw error;
        });
    }
    async getTypingsForDependencies() {
        const { dependencies, devDependencies, peerDependencies } = await this.getDependencies();
        const allDependencies = new Set([
            ...Object.keys(dependencies || {}),
            ...Object.keys(devDependencies || {}),
            ...Object.keys(peerDependencies || {}),
            "framer",
            "csstype",
            "prop-types",
        ]);
        const files = [];
        allDependencies.forEach(name => {
            if (name.startsWith("@types"))
                return;
            const dts = this.getTypingsForPackage(name);
            files.push(...dts);
        });
        try {
            const motionDtsPath = path_1.default.join(this.path, "node_modules", "framer", "node_modules", "framer-motion", "dist", "framer-motion.d.ts");
            const stat = await fs_extra_1.default.stat(motionDtsPath);
            if (stat.isFile() || stat.isSymbolicLink()) {
                files.push(...this.getTypingsAroundMain("framer-motion", motionDtsPath));
            }
        }
        catch (e) {
            if (!e.message.includes("ENOENT"))
                throw e;
        }
        return files;
    }
    getTypingsForPackage(name) {
        const code = this.path;
        try {
            const atTypesIndex = path_1.default.join(code, "node_modules/@types", name, "index.d.ts");
            const stat = fs_extra_1.default.statSync(atTypesIndex);
            if (stat.isFile() || stat.isSymbolicLink()) {
                return this.getTypingsAroundMain(name, atTypesIndex);
            }
        }
        catch (e) {
            if (!e.message.includes("ENOENT"))
                throw e;
        }
        try {
            const moduleIndex = path_1.default.join(code, "node_modules", name, "index.d.ts");
            const stat = fs_extra_1.default.statSync(moduleIndex);
            if (stat.isFile() || stat.isSymbolicLink()) {
                return this.getTypingsAroundMain(name, moduleIndex);
            }
        }
        catch (e) {
            if (!e.message.includes("ENOENT"))
                throw e;
        }
        try {
            const pkg = JSON.parse(fs_extra_1.default.readFileSync(path_1.default.join(code, "node_modules", name, "package.json"), "utf8"));
            let types = null;
            if (pkg.typescript && pkg.typescript.definition) {
                types = pkg.typescript.definition;
            }
            else if (pkg.types) {
                types = pkg.types;
            }
            else if (pkg.typings) {
                types = pkg.typings;
            }
            else if (pkg.main) {
                types = pkg.main.slice(0, -3) + ".d.ts";
            }
            if (types) {
                const modulePath = path_1.default.join(code, "node_modules", name, types);
                const stat = fs_extra_1.default.statSync(modulePath);
                if (stat.isFile() || stat.isSymbolicLink()) {
                    return this.getTypingsAroundMain(name, modulePath);
                }
            }
        }
        catch (e) {
            if (!e.message.includes("ENOENT"))
                throw e;
        }
        return [];
    }
    getTypingsAroundMain(module, modulepath) {
        const modulecontent = fs_extra_1.default.readFileSync(modulepath, "utf8");
        const res = [{ module: path_1.default.join(module, "index.d.ts"), content: modulecontent }];
        const base = path_1.default.dirname(modulepath);
        const scandir = (dirname = "") => {
            fs_extra_1.default.readdirSync(path_1.default.join(base, dirname)).forEach(filename => {
                if (filename.startsWith("."))
                    return;
                if (filename === "node_modules")
                    return;
                if (module === "react" && filename === "global.d.ts")
                    return;
                const filepath = path_1.default.join(base, dirname, filename);
                if (filepath === modulepath)
                    return;
                const stat = fs_extra_1.default.statSync(filepath);
                if (stat.isDirectory()) {
                    scandir(path_1.default.join(dirname, filename));
                    return;
                }
                if (!filepath.endsWith(".d.ts"))
                    return;
                res.push({
                    module: path_1.default.join(module, dirname, filename),
                    content: fs_extra_1.default.readFileSync(filepath, "utf8"),
                });
            });
        };
        scandir("");
        if (res.length > 1 && !modulepath.endsWith("index.d.ts")) {
            // the module might have internal depedencies using the original filename
            res.push({ module: path_1.default.join(module, path_1.default.basename(modulepath)), content: modulecontent });
        }
        return res;
    }
    setYarnEnv(env) {
        this.pkg.setYarnEnv(env);
    }
}
exports.Project = Project;
/** Add live canvas data to the CanvasStore of a build. */
function injectCanvasData(fileContent, canvasData = '{"children":[]}') {
    // Search for the last entry, in case some external framer packages didn't replace their CANVAS_DATA
    const needle1 = "CanvasStore.shared(";
    const needle2 = "); // CANVAS_DATA;\n";
    let at = fileContent.lastIndexOf(needle1 + needle2);
    if (at < 0) {
        return fileContent;
    }
    at += needle1.length;
    // always remove the CANVAS_DATA marker
    return fileContent.slice(0, at) + canvasData + ");\n" + fileContent.slice(at + needle2.length);
}
// Sanity checks to make sure project is a valid Framer project
async function validateIsFramerProject(projectPath) {
    assert_1.default.strict((await fs_extra_1.default.stat(projectPath)).isDirectory(), "Project is not a valid Framer project");
    return projectPath;
}
