"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const resolve_pkg_1 = __importDefault(require("resolve-pkg"));
const split_1 = __importDefault(require("split"));
const Logger_1 = require("../Logger");
const types_1 = require("../types");
const logger = new Logger_1.Logger("framer-build-library:yarn");
// Reading from the `process` is expensive and
// should be done as few times as possible
const cwd = process.cwd();
const nodePath = process.execPath;
// 1. Find yarn dependency path in a parent directory
// 2. Follow the bin.yarn path from the package.json field
const yarnPackagePath = resolve_pkg_1.default("yarn", { cwd: __dirname });
if (!yarnPackagePath) {
    throw new Error("Could not find Yarn executable.");
}
exports.yarnPath = path_1.default.resolve(yarnPackagePath, JSON.parse(fs_extra_1.default.readFileSync(path_1.default.join(yarnPackagePath, "package.json"), "utf8")).bin.yarn);
var YarnCommands;
(function (YarnCommands) {
    YarnCommands["add"] = "add";
    YarnCommands["check"] = "check";
    YarnCommands["info"] = "info";
    YarnCommands["install"] = "install";
    YarnCommands["link"] = "link";
    YarnCommands["list"] = "list";
    // outdated = "outdated",
    // pack = "pack",
    YarnCommands["publish"] = "publish";
    YarnCommands["remove"] = "remove";
    YarnCommands["unlink"] = "unlink";
    YarnCommands["upgrade"] = "upgrade";
    // version = "version",
})(YarnCommands = exports.YarnCommands || (exports.YarnCommands = {}));
var YarnDependencyTypes;
(function (YarnDependencyTypes) {
    YarnDependencyTypes["dep"] = "dependency";
    YarnDependencyTypes["peer"] = "peer";
    YarnDependencyTypes["dev"] = "dev";
    YarnDependencyTypes["dependency"] = "dependency";
    YarnDependencyTypes["peerDependency"] = "peer";
    YarnDependencyTypes["devDependency"] = "dev";
})(YarnDependencyTypes = exports.YarnDependencyTypes || (exports.YarnDependencyTypes = {}));
// Yarn methods are lazily built and stored on the
// Yarn class under the command name (using a Proxy)
class Yarn {
    constructor({ defaults = ["--json", "--no-progress"] } = {}) {
        this.env = {};
        this.defaults = defaults;
    }
    async add(packageName, version, type = YarnDependencyTypes.dependency) {
        const dependencyQuery = version ? `${packageName}@${version}` : packageName;
        return this.run(exports.yarnPath, YarnCommands.add, dependencyQuery, `--${type}`);
    }
    async check() {
        return this.run(exports.yarnPath, YarnCommands.check);
    }
    info(packageName, info) {
        return this.run(exports.yarnPath, YarnCommands.info, packageName, info);
    }
    install(...args) {
        return this.run(exports.yarnPath, YarnCommands.install, ...args);
    }
    link(packageName) {
        return this.run(exports.yarnPath, YarnCommands.link, packageName);
    }
    publish(...args) {
        return this.run(exports.yarnPath, YarnCommands.publish, ...args);
    }
    remove(dependency) {
        return this.run(exports.yarnPath, YarnCommands.remove, dependency);
    }
    unlink(packageName) {
        return this.run(exports.yarnPath, YarnCommands.unlink, packageName);
    }
    list(...args) {
        return this.run(exports.yarnPath, YarnCommands.list, ...args);
    }
    upgrade(packageName, version) {
        const dependencyQuery = version ? `${packageName}@${version}` : packageName;
        return this.run(exports.yarnPath, YarnCommands.upgrade, dependencyQuery);
    }
    run(yarnBinPath, command, ...args) {
        return new Promise((resolve, reject) => {
            logger.info("yarn", command, ...args, ...this.defaults);
            const env = { ...process.env, ...this.env };
            const childProcess = child_process_1.spawn(nodePath, [yarnBinPath, command, ...args, ...this.defaults], { cwd, env });
            const result = {
                type: "success",
                [types_1.YarnCommandResponseTypes.error]: [],
                [types_1.YarnCommandResponseTypes.info]: [],
                [types_1.YarnCommandResponseTypes.inspect]: [],
                [types_1.YarnCommandResponseTypes.success]: [],
                [types_1.YarnCommandResponseTypes.tree]: [],
                [types_1.YarnCommandResponseTypes.warning]: [],
                output: [],
            };
            parseJSONStream(childProcess.stdout, result);
            parseJSONStream(childProcess.stderr, result);
            const onExit = (code) => {
                resolve(processYarnCommandResult(result, code));
            };
            const onError = () => {
                // The 'exit' event may or may not fire after an error has occurred.
                // So guard against accidentally invoking handler functions multiple times.
                childProcess.off("exit", onExit);
                reject(processYarnCommandResult(result, 1));
            };
            childProcess.on("error", onError);
            childProcess.on("exit", onExit);
        });
    }
    setEnv(env) {
        this.env = env;
    }
}
exports.Yarn = Yarn;
function processYarnCommandResult(result, code) {
    const { error, output } = result;
    logger.log(output);
    if (error.length) {
        logger.error(error);
    }
    result.type = error.length === 0 && !code ? "success" : "failure";
    return result;
}
function parseJSONStream(stream, result) {
    return (stream
        // outdated typedefs for "split"
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        .pipe(split_1.default(JSON.parse, null, { trailing: false }))
        .on("data", function (output) {
        // Each chunk now is a JS object
        const { type } = output;
        // Only keep response types we are tracking
        if (!result[types_1.YarnCommandResponseTypes[type]])
            return;
        // Track message by type and as part of overall output
        result.output.push(output);
        result[types_1.YarnCommandResponseTypes[type]].push(output);
    })
        .on("error", function (error) {
        // Syntax errors will land here
        // Note, this ends the stream.
        result[types_1.YarnCommandResponseTypes.error].push({
            type: types_1.YarnCommandResponseTypes.error,
            data: error,
        });
        // Since stream is not continuing, emit "close"
        stream.emit("close");
    }));
}
